
from __future__ import absolute_import, division, print_function, unicode_literals

import numpy as np
import pytest
import xarray as xr

from cf2cdm import cfcoords


@pytest.fixture
def da1():
    latitude = [0., 0.5]
    longitude = [10., 10.5]
    time = ['2017-12-01T00:00:00', '2017-12-01T12:00:00', '2017-12-02T00:00:00']
    level = [950, 500]
    data = xr.DataArray(
        np.zeros((2, 2, 3, 2), dtype='float32'),
        coords=[
            ('lat', latitude, {'units': 'degrees_north'}),
            ('lon', longitude, {'units': 'degrees_east'}),
            ('ref_time', np.array(time, dtype=np.datetime64),
                {'standard_name': 'forecast_reference_time'}),
            ('level', np.array(level), {'units': 'hPa'}),
        ])
    return data


@pytest.fixture
def da2():
    latitude = [0., 0.5]
    longitude = [10., 10.5]
    time = ['2017-12-01T00:00:00', '2017-12-01T12:00:00', '2017-12-02T00:00:00']
    level = [950, 500]
    data = xr.DataArray(
        np.zeros((2, 2, 3, 2), dtype='float32'),
        coords=[
            ('lat', latitude, {'units': 'degrees_north'}),
            ('lon', longitude, {'units': 'degrees_east'}),
            ('time', np.array(time, dtype=np.datetime64)),
            ('level', np.array(level), {'units': 'hPa'}),
        ])
    return data


@pytest.fixture
def dataarray3():
    latitude = [0., 0.5]
    longitude = [10., 10.5]
    step = [0, 24, 48]
    time = ['2017-12-01T00:00:00']
    level = [950, 500]
    data = xr.DataArray(
        np.zeros((2, 2, 3, 1, 2), dtype='float32'),
        coords=[
            ('lat', latitude, {'units': 'degrees_north'}),
            ('lon', longitude, {'units': 'degrees_east'}),
            ('step', np.array(step, dtype=np.timedelta64), {'standard_name': 'forecast_period'}),
            ('ref_time', np.array(time, dtype=np.datetime64),
                {'standard_name': 'forecast_reference_time'}),
            ('level', np.array(level), {'units': 'hPa'}),
        ])

    return data


def test_match_values():
    mapping = {'callable': len, 'int': 1}
    res = cfcoords.match_values(callable, mapping)

    assert res == ['callable']


def test_coord_translator(da1):
    res = cfcoords.coord_translator('level', 'hPa', lambda x: False, 'level', da1)
    assert da1.equals(res)

    with pytest.raises(ValueError):
        cfcoords.coord_translator('level', 'hPa', lambda x: True, 'level', da1)

    res = cfcoords.coord_translator('level', 'hPa', cfcoords.is_vertical_pressure, 'level', da1)
    assert da1.equals(res)

    with pytest.raises(ValueError):
        cfcoords.coord_translator('level', 'hPa', cfcoords.is_latitude, 'level', da1)

    res = cfcoords.coord_translator('level', 'Pa', cfcoords.is_vertical_pressure, 'level', da1)
    assert not da1.equals(res)

    res = cfcoords.coord_translator('step', 'h', cfcoords.is_forecast_period, 'step', da1)
    assert da1.equals(res)


def test_translate_coords(da1, da2):
    result = cfcoords.translate_coords(da1)

    assert 'latitude' in result.coords
    assert 'longitude' in result.coords
    assert 'time' in result.coords

    result = cfcoords.translate_coords(da2)

    assert 'latitude' in result.coords
    assert 'longitude' in result.coords
    assert 'valid_time' in result.coords


def test_ensure_valid_time(da1, dataarray3):
    result1 = cfcoords.ensure_valid_time(da1.squeeze())
    result2 = cfcoords.ensure_valid_time(result1)

    assert 'valid_time' in result1.coords
    assert result2 is result1

    result1 = cfcoords.ensure_valid_time(dataarray3.squeeze())
    result2 = cfcoords.ensure_valid_time(result1)

    assert 'valid_time' in result1.coords
    assert result2 is result1




import os.path

import pytest

from cfgrib import xarray_store
from cfgrib import xarray_to_grib

SAMPLE_DATA_FOLDER = os.path.join(os.path.dirname(__file__), 'sample-data')


@pytest.mark.parametrize('grib_name', [
    'era5-levels-members',
    pytest.param('hpa_and_pa', marks=pytest.mark.xfail),
    'fields_with_missing_values',
    'lambert_grid',
    'reduced_gg',
    'regular_gg_sfc',
    'regular_gg_pl',
    'regular_gg_ml',
    'regular_gg_ml_g2',
    'regular_ll_sfc',
    'regular_ll_msl',
    'scanning_mode_64',
    'spherical_harmonics',
    't_analysis_and_fc_0',
])
def test_open_dataset(grib_name):
    grib_path = os.path.join(SAMPLE_DATA_FOLDER, grib_name + '.grib')
    res = xarray_store.open_dataset(grib_path, cache=False)
    print(res)


@pytest.mark.parametrize('grib_name', [
    't_on_different_level_types',
    'tp_on_different_grid_resolutions',
    pytest.param('uv_on_different_levels', marks=pytest.mark.xfail),
])
def test_open_dataset_fail(grib_name):
    grib_path = os.path.join(SAMPLE_DATA_FOLDER, grib_name + '.grib')
    with pytest.raises(ValueError):
        xarray_store.open_dataset(grib_path, cache=False)


# writing eccodes flavor is not supported ATM
@pytest.mark.parametrize('grib_name', [
    pytest.param('era5-levels-members', marks=pytest.mark.xfail),
    pytest.param('hpa_and_pa', marks=pytest.mark.xfail),
    pytest.param('fields_with_missing_values', marks=pytest.mark.xfail),
    pytest.param('lambert_grid', marks=pytest.mark.xfail),
    pytest.param('reduced_gg', marks=pytest.mark.xfail),
    pytest.param('regular_gg_sfc', marks=pytest.mark.xfail),
    pytest.param('regular_gg_pl', marks=pytest.mark.xfail),
    pytest.param('regular_gg_ml', marks=pytest.mark.xfail),
    pytest.param('regular_gg_ml_g2', marks=pytest.mark.xfail),
    pytest.param('regular_ll_sfc', marks=pytest.mark.xfail),
    pytest.param('regular_ll_msl', marks=pytest.mark.xfail),
    pytest.param('scanning_mode_64', marks=pytest.mark.xfail),
    pytest.param('spherical_harmonics', marks=pytest.mark.xfail),
    pytest.param('t_analysis_and_fc_0', marks=pytest.mark.xfail),
])
def test_to_grib_eccodes(grib_name, tmpdir):
    grib_path = os.path.join(SAMPLE_DATA_FOLDER, grib_name + '.grib')
    out_path = str(tmpdir.join(grib_name + '.grib'))

    res = xarray_store.open_dataset(grib_path, flavour_name='eccodes', cache=False)

    xarray_to_grib.canonical_dataset_to_grib(res, out_path)
    reread = xarray_store.open_dataset(out_path, flavour_name='eccodes', cache=False)
    assert res.equals(reread)


@pytest.mark.parametrize('grib_name', [
    'era5-levels-members',
    pytest.param('hpa_and_pa', marks=pytest.mark.xfail),
    pytest.param('fields_with_missing_values', marks=pytest.mark.xfail),
    pytest.param('lambert_grid', marks=pytest.mark.xfail),
    pytest.param('reduced_gg', marks=pytest.mark.xfail),
    pytest.param('regular_gg_sfc', marks=pytest.mark.xfail),
    pytest.param('regular_gg_pl', marks=pytest.mark.xfail),
    pytest.param('regular_gg_ml', marks=pytest.mark.xfail),
    pytest.param('regular_gg_ml_g2', marks=pytest.mark.xfail),
    'regular_ll_sfc',
    'regular_ll_msl',
    'scanning_mode_64',
    pytest.param('spherical_harmonics', marks=pytest.mark.xfail),
    't_analysis_and_fc_0',
])
def test_to_grib_ecmwf(grib_name, tmpdir):
    grib_path = os.path.join(SAMPLE_DATA_FOLDER, grib_name + '.grib')
    out_path = str(tmpdir.join(grib_name + '.grib'))

    res = xarray_store.open_dataset(grib_path, cache=False)

    xarray_to_grib.canonical_dataset_to_grib(res, out_path)
    reread = xarray_store.open_dataset(out_path, cache=False)
    assert res.equals(reread)
